'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _apiBased = require('../api-based');

var _apiBased2 = _interopRequireDefault(_apiBased);

var _esNext = require('./es-next');

var _esNext2 = _interopRequireDefault(_esNext);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TypeScriptTestFileCompiler = function (_APIBasedTestFileComp) {
    (0, _inherits3.default)(TypeScriptTestFileCompiler, _APIBasedTestFileComp);

    function TypeScriptTestFileCompiler() {
        (0, _classCallCheck3.default)(this, TypeScriptTestFileCompiler);
        return (0, _possibleConstructorReturn3.default)(this, _APIBasedTestFileComp.apply(this, arguments));
    }

    TypeScriptTestFileCompiler._getTypescriptOptions = function _getTypescriptOptions() {
        // NOTE: lazy load the compiler
        var ts = require('typescript');

        return {
            allowJs: true,
            pretty: true,
            inlineSourceMap: true,
            noEmit: true,
            noImplicitAny: false,
            module: ts.ModuleKind.CommonJS,
            target: 'ES6',
            lib: ['lib.es6.d.ts'],
            baseUrl: __dirname,
            paths: { testcafe: ['../../../../ts-defs/index.d.ts'] }
        };
    };

    TypeScriptTestFileCompiler._reportErrors = function _reportErrors(diagnostics) {
        // NOTE: lazy load the compiler
        var ts = require('typescript');
        var errMsg = 'TypeScript compilation failed.\n';

        diagnostics.forEach(function (d) {
            var file = d.file;

            var _file$getLineAndChara = file.getLineAndCharacterOfPosition(d.start),
                line = _file$getLineAndChara.line,
                character = _file$getLineAndChara.character;

            var message = ts.flattenDiagnosticMessageText(d.messageText, '\n');

            errMsg += file.fileName + ' (' + (line + 1) + ', ' + (character + 1) + '): ' + message + '\n';
        });

        throw new Error(errMsg);
    };

    TypeScriptTestFileCompiler.prototype._compileCode = function _compileCode(code, filename) {
        // NOTE: lazy load the compiler
        var ts = require('typescript');

        if (this.cache[filename]) return this.cache[filename];

        var opts = TypeScriptTestFileCompiler._getTypescriptOptions();
        var program = ts.createProgram([filename], opts);
        var diagnostics = ts.getPreEmitDiagnostics(program);

        if (diagnostics.length) TypeScriptTestFileCompiler._reportErrors(diagnostics);

        var result = ts.transpileModule(code, {
            compilerOptions: opts,
            fileName: filename,
            renamedDependencies: { testcafe: _apiBased2.default.EXPORTABLE_LIB_PATH }
        });

        this.cache[filename] = result;

        return result.outputText;
    };

    TypeScriptTestFileCompiler.prototype._getRequireCompilers = function _getRequireCompilers() {
        var _this2 = this;

        return {
            '.ts': function ts(code, filename) {
                return _this2._compileCode(code, filename);
            },
            '.js': function js(code, filename) {
                return _esNext2.default.prototype._compileCode.call(_this2, code, filename);
            }
        };
    };

    TypeScriptTestFileCompiler.prototype.getSupportedExtension = function getSupportedExtension() {
        return '.ts';
    };

    return TypeScriptTestFileCompiler;
}(_apiBased2.default);

exports.default = TypeScriptTestFileCompiler;
module.exports = exports['default'];